import "package:flutter/material.dart";
import 'package:verttice_ui/src/widgets/border_label.dart';
import 'forms_factory_index.dart';

class ColorSelectorFormFactory extends StatefulWidget {
  final Function(String name, dynamic value) onChanged;
  final String displayName;
  final String name;
  final SelectorColors? value;
  final SelectorColors? defaultValue;
  final List<SelectorColors> colors;
  final FormFieldState<dynamic>? state;

  const ColorSelectorFormFactory({
    Key? key,
    required this.onChanged,
    required this.displayName,
    required this.name,
    required this.colors,
    this.defaultValue,
    this.value,
    this.state,
  }) : super(key: key);

  @override
  _CheckboxFormState createState() => _CheckboxFormState();
}

class _CheckboxFormState extends State<ColorSelectorFormFactory> {
  SelectorColors? _value;

  @override
  void initState() {
    _value = widget.value ?? widget.defaultValue;
    super.initState();
  }

  @override
  void didUpdateWidget(covariant ColorSelectorFormFactory oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (_value != widget.value) {
      _value = widget.value;
    }
  }

  Color _getColorFromValue(SelectorColors? value) {
    switch (value) {
      case SelectorColors.green:
        return Colors.green;
      case SelectorColors.red:
        return Colors.red;
      case SelectorColors.orange:
        return Colors.orange;
      case SelectorColors.grey:
        return Colors.grey;
      case SelectorColors.yellow:
        return Colors.yellow;
      default:
        return Colors.white;
    }
  }

  @override
  Widget build(BuildContext context) {
    return BorderLabel(
      label: widget.displayName,
      child: Container(
        height: 55,
        decoration: BoxDecoration(
          borderRadius: BorderRadius.circular(5),
          color: _getColorFromValue(_value),
        ),
        child: Center(
          child: Row(
            crossAxisAlignment: CrossAxisAlignment.center,
            mainAxisAlignment: MainAxisAlignment.center,
            children: widget.colors
                .map(
                  (e) => InkWell(
                    onTap: () {
                      widget.onChanged(widget.name, e.name);
                    },
                    child: Padding(
                      padding: const EdgeInsets.symmetric(horizontal: 3),
                      child: CircleAvatar(
                        child: _value == e
                            ? SizedBox(
                                height: 20,
                                width: 20,
                                child: CircleAvatar(
                                  backgroundColor: _getColorFromValue(e),
                                ),
                              )
                            : null,
                        backgroundColor:
                            _value == e ? null : _getColorFromValue(e),
                      ),
                    ),
                  ),
                )
                .toList(),
          ),
        ),
      ),
    );
  }
}
