import "package:collection/collection.dart";
import 'package:verttice_logic/verttice_logic.dart';
import 'package:verttice_ui/src/const/box_sizes.dart';
import "package:verttice_ui/src/const/icons.dart";
import 'package:day_night_time_picker/day_night_time_picker.dart';
import 'package:day_night_time_picker/lib/constants.dart';
import "package:flutter/material.dart";
import 'package:flutter/services.dart';
import 'package:verttice_ui/src/widgets/border_label.dart';
import 'package:verttice_ui/src/widgets/themed_widgets/themed_widgets_index.dart';

class DateTimeRangeFormFactory extends StatefulWidget {
  final Function(String name, dynamic value) onChanged;
  final String displayName;
  final String name;
  final dynamic value;
  final FormFieldState<dynamic>? state;

  const DateTimeRangeFormFactory({
    Key? key,
    required this.onChanged,
    required this.displayName,
    required this.name,
    required this.value,
    this.state,
  }) : super(key: key);

  @override
  _DateTimeRangeFormFactoryState createState() =>
      _DateTimeRangeFormFactoryState();
}

class _DateTimeRangeFormFactoryState extends State<DateTimeRangeFormFactory> {
  var _value = <String, int?>{};

  @override
  void initState() {
    super.initState();
    _value = DataClassUtils.dynamicToMapStringIntOrNull(widget.value)
        .cast<String, int?>();
  }

  @override
  void didUpdateWidget(covariant DateTimeRangeFormFactory oldWidget) {
    super.didUpdateWidget(oldWidget);
    final value = DataClassUtils.dynamicToMapStringIntOrNull(widget.value);
    if (!const MapEquality().equals(_value, value)) {
      _value = DataClassUtils.dynamicToMapStringIntOrNull(widget.value)
          .cast<String, int?>();
    }
  }

  @override
  Widget build(BuildContext context) {
    return BorderLabel(
      label: widget.displayName,
      onCopied: () async {
        await Clipboard.setData(
          ClipboardData(
            text:
                "${_value["startDate"] != null && _value["startDate"] is int ? Dates.formatDataAndTime(miliseconds: _value["startDate"]!) : ""} - ${_value["endDate"] != null && _value["endDate"] is int ? Dates.formatDataAndTime(miliseconds: _value["endDate"]!) : ""}",
          ),
        );
        ThemedToast.info(
          context: context,
          message: "Copied to clipboard",
        );
      },
      actions: [
        PhysicalModel(
          color: Colors.grey,
          elevation: 5,
          shape: BoxShape.circle,
          child: CircleAvatar(
            backgroundColor: Colors.red,
            child: InkWell(
              child: ThemedIcons.clear(
                iconStyle: IconStyle(
                  color: Colors.white,
                ),
              ),
              onTap: () {
                widget.onChanged(widget.name, null);
                widget.state?.didChange(null);
              },
            ),
          ),
        ),
        sizedBoxW2,
      ],
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        mainAxisAlignment: MainAxisAlignment.start,
        children: [
          Row(
            children: [
              TextButton.icon(
                icon: ThemedIcons.calendar(),
                label: Text(
                  _value["startDate"] != null && _value["startDate"] is int
                      ? Dates.formatDataAndTime(
                          miliseconds: _value["startDate"]!,
                          time: false,
                        )
                      : "Start Date",
                ),
                onPressed: () async {
                  DateTime? initialDate;
                  if (_value['startDate'] != null &&
                      _value['startDate'] is int) {
                    initialDate = DateTime.fromMillisecondsSinceEpoch(
                      _value['startDate']!,
                    );
                  }
                  DateTime? maxDate;
                  if (_value['endDate'] != null && _value['endDate'] is int) {
                    maxDate =
                        DateTime.fromMillisecondsSinceEpoch(_value['endDate']!);
                  }

                  DateTime? date = await showDatePicker(
                    context: context,
                    initialDate: initialDate ?? DateTime.now(),
                    firstDate: DateTime(2000),
                    lastDate: maxDate ?? DateTime(2100),
                  );

                  if (date == null) {
                    return;
                  }
                  date = date.toUtc();

                  final choosenDate = {
                    ..._value,
                    "startDate": date.millisecondsSinceEpoch,
                    if (maxDate == null ||
                        (maxDate.day == date.day &&
                            maxDate.month == date.month &&
                            maxDate.year == date.year))
                      "endDate": DateTime(
                        date.year,
                        date.month,
                        date.day,
                        date.hour + 3,
                        date.minute,
                      ).millisecondsSinceEpoch,
                  };
                  widget.onChanged(widget.name, choosenDate);
                  widget.state?.didChange(choosenDate);
                },
              ),
              if (_value["startDate"] != null && _value["startDate"] is int)
                TextButton.icon(
                  icon: ThemedIcons.clock(),
                  label: Text(
                    _value["startDate"] != null && _value["startDate"] is int
                        ? Dates.formatDataAndTime(
                            miliseconds: _value["startDate"]!,
                            date: false,
                          )
                        : "Start Time",
                  ),
                  onPressed: () async {
                    DateTime? maxDate;
                    if (_value['endDate'] != null && _value['endDate'] is int) {
                      maxDate = DateTime.fromMillisecondsSinceEpoch(
                          _value['endDate']!);
                    }
                    await Navigator.of(context).push(
                      showPicker(
                        context: context,
                        is24HrFormat: true,
                        minuteInterval: MinuteInterval.FIVE,
                        value: TimeOfDay.fromDateTime(
                          DateTime.fromMillisecondsSinceEpoch(
                            _value['startDate']!,
                          ),
                        ),
                        onChange: (time) {
                          final currentDate =
                              DateTime.fromMillisecondsSinceEpoch(
                            _value['startDate']!,
                          );
                          DateTime date = DateTime(
                            currentDate.year,
                            currentDate.month,
                            currentDate.day,
                            time.hour,
                            time.minute,
                          ).toUtc();

                          final choosenDate = {
                            ..._value,
                            "startDate": date.millisecondsSinceEpoch,
                            if (maxDate == null ||
                                (maxDate.day == date.day &&
                                    maxDate.month == date.month &&
                                    maxDate.year == date.year))
                              "endDate": DateTime(
                                date.year,
                                date.month,
                                date.day,
                                date.hour + 3,
                                date.minute,
                              ).millisecondsSinceEpoch,
                          };
                          widget.onChanged(widget.name, choosenDate);
                          widget.state?.didChange(choosenDate);
                        },
                      ),
                    );
                  },
                ),
            ],
          ),
          if (_value['endDate'] != null && _value['endDate'] is int)
            Row(
              children: [
                TextButton.icon(
                  icon: ThemedIcons.calendar(),
                  label: Text(
                    _value["endDate"] != null && _value["endDate"] is int
                        ? Dates.formatDataAndTime(
                            miliseconds: _value["endDate"]!,
                            time: false,
                          )
                        : "End Date",
                  ),
                  onPressed: () async {
                    DateTime? initialDate;
                    if (_value['endDate'] != null && _value['endDate'] is int) {
                      initialDate = DateTime.fromMillisecondsSinceEpoch(
                        _value['endDate']!,
                      );
                    }
                    DateTime? minDate;
                    if (_value['startDate'] != null &&
                        _value['startDate'] is int) {
                      minDate = DateTime.fromMillisecondsSinceEpoch(
                          _value['startDate']!);
                    }

                    DateTime? date = await showDatePicker(
                      context: context,
                      initialDate: initialDate ?? DateTime.now(),
                      firstDate: minDate ?? DateTime(2000),
                      lastDate: DateTime(2100),
                    );

                    if (date == null) {
                      return;
                    }
                    date = date.toUtc();
                    DateTime dateToUpdate = DateTime(
                      date.year,
                      date.month,
                      date.day,
                      initialDate?.hour ?? date.hour,
                      initialDate?.minute ?? date.minute,
                    );

                    final choosenDate = {
                      ..._value,
                      "endDate": dateToUpdate.millisecondsSinceEpoch,
                    };
                    widget.onChanged(widget.name, choosenDate);
                    widget.state?.didChange(choosenDate);
                  },
                ),
                if (_value["endDate"] != null && _value["endDate"] is int)
                  TextButton.icon(
                    icon: ThemedIcons.clock(),
                    label: Text(
                      _value["endDate"] != null && _value["endDate"] is int
                          ? Dates.formatDataAndTime(
                              miliseconds: _value["endDate"]!,
                              date: false,
                            )
                          : "End Time",
                    ),
                    onPressed: () async {
                      DateTime? minDate;
                      if (_value['startDate'] != null &&
                          _value['startDate'] is int) {
                        DateTime? maxDate;
                        if (_value['endDate'] != null &&
                            _value['endDate'] is int) {
                          maxDate = DateTime.fromMillisecondsSinceEpoch(
                            _value['endDate']!,
                          );
                        }
                        minDate = DateTime.fromMillisecondsSinceEpoch(
                          _value['startDate']!,
                        );
                        if (maxDate != null &&
                            ((maxDate.day != minDate.day) ||
                                (maxDate.day == minDate.day &&
                                    (maxDate.month != minDate.month ||
                                        maxDate.year != minDate.year)))) {
                          minDate = null;
                        }
                      }

                      await Navigator.of(context).push(
                        showPicker(
                          context: context,
                          is24HrFormat: true,
                          minHour: minDate?.hour.toDouble() ?? double.infinity,
                          minuteInterval: MinuteInterval.FIVE,
                          value: TimeOfDay.fromDateTime(
                            DateTime.fromMillisecondsSinceEpoch(
                              _value['endDate'] ?? _value["startDate"]!,
                            ),
                          ),
                          onChange: (time) {
                            final currentDate =
                                DateTime.fromMillisecondsSinceEpoch(
                              _value['endDate'] ?? _value["startDate"]!,
                            );
                            DateTime date = DateTime(
                              currentDate.year,
                              currentDate.month,
                              currentDate.day,
                              time.hour,
                              time.minute,
                            ).toUtc();

                            final choosenDate = {
                              ..._value,
                              "endDate": date.millisecondsSinceEpoch,
                            };
                            widget.onChanged(widget.name, choosenDate);
                            widget.state?.didChange(choosenDate);
                          },
                        ),
                      );
                    },
                  ),
              ],
            ),
        ],
      ),
    );
  }
}
