import "dart:math";
import "package:verttice_ui/src/const/box_sizes.dart";
import "package:verttice_ui/src/const/icons.dart";
import "package:flutter/material.dart";
import 'package:flutter/services.dart';
import 'package:verttice_ui/src/widgets/border_label.dart';
import 'package:verttice_ui/src/widgets/themed_widgets/themed_widgets_index.dart';

class MultilineTextFormFactory extends StatefulWidget {
  final String displayName;
  final String name;
  final String value;
  final FormFieldState<dynamic>? state;

  final Function(String name, dynamic value) onChanged;

  const MultilineTextFormFactory({
    Key? key,
    required this.displayName,
    required this.name,
    required this.onChanged,
    required this.value,
    this.state,
  }) : super(key: key);
  @override
  _MultilineTextFormFactoryState createState() =>
      _MultilineTextFormFactoryState();
}

class _MultilineTextFormFactoryState extends State<MultilineTextFormFactory> {
  TextEditingController? _controller;
  String _value = "";
  int _baseOffset = 0;
  late FocusNode _focusNode;
  @override
  void initState() {
    super.initState();
    _value = widget.value;
    _controller = TextEditingController(
      text: _value,
    );
    _controller!.selection = TextSelection.fromPosition(
      TextPosition(
        offset: min(
          _baseOffset,
          _controller != null ? _controller!.text.length : 0,
        ),
      ),
    );

    _controller!.addListener(_listen);
    _focusNode = FocusNode();
  }

  @override
  void dispose() {
    _focusNode.dispose();
    _controller!.dispose();
    super.dispose();
  }

  @override
  void didUpdateWidget(covariant MultilineTextFormFactory oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (_value != widget.value) {
      _value = widget.value;
      _controller = TextEditingController(
        text: _value,
      );
      _controller!.selection = TextSelection.fromPosition(
        TextPosition(
          offset: min(
            _baseOffset,
            _controller != null ? _controller!.text.length : 0,
          ),
        ),
      );
      _controller!.addListener(_listen);
    }
  }

  void _listen() {
    setState(() {
      _baseOffset = _controller!.selection.baseOffset;
    });
  }

  @override
  Widget build(BuildContext context) {
    return BorderLabel(
      onCopied: () async {
        await Clipboard.setData(
          ClipboardData(
            text: _value,
          ),
        );
        ThemedToast.info(
          context: context,
          message: "Copied to clipboard",
        );
      },
      label: widget.displayName,
      onPressed: () async {
        await showDialog(
          context: context,
          useSafeArea: true,
          builder: (BuildContext context) {
            return SimpleDialog(
              insetPadding: const EdgeInsets.all(6),
              contentPadding: const EdgeInsets.all(6),
              title: SelectableText(widget.displayName),
              children: [
                sizedBoxH4,
                SizedBox(
                  height: 300,
                  width: 400,
                  child: Center(
                    child: Padding(
                      padding: const EdgeInsets.all(16),
                      child: TextFormField(
                        cursorColor: Colors.amberAccent,
                        onChanged: (val) {
                          setState(() {
                            _value = val;
                          });
                          widget.onChanged(widget.name, val);
                          widget.state?.didChange(val);
                        },
                        minLines: 10,
                        maxLines: 20,
                        controller: _controller,
                        keyboardType: TextInputType.multiline,
                        textDirection: TextDirection.ltr,
                        autofocus: false,
                        decoration: InputDecoration(
                          suffixIcon: IconButton(
                            onPressed: () {
                              _controller!.clear();
                              widget.onChanged(widget.name, "");
                              widget.state?.didChange("");
                            },
                            icon: ThemedIcons.clear(
                              iconStyle: IconStyle(
                                color: Colors.redAccent,
                              ),
                            ),
                          ),
                        ),
                      ),
                    ),
                  ),
                ),
                sizedBoxH4,
                const Divider(),
                TextButton(
                  child: const Text(
                    "Finish",
                    style: TextStyle(color: Colors.red),
                  ),
                  onPressed: () async {
                    return Navigator.of(context).pop();
                  },
                ),
              ],
            );
          },
        );
      },
      child: LayoutBuilder(builder: (context, constraints) {
        return SizedBox(
          width: constraints.maxWidth,
          child: Tooltip(
            message: _value,
            child: Text(
              _value,
              maxLines: 3,
              style: const TextStyle(
                overflow: TextOverflow.fade,
              ),
            ),
          ),
        );
      }),
    );
  }
}
