import "package:collection/collection.dart";
import "package:flutter/material.dart";
import "package:dropdown_search/dropdown_search.dart";
import 'package:verttice_ui/src/widgets/border_label.dart';
import 'package:verttice_ui/src/widgets/data_factory/common/utils.dart';

class SelectFormFactory extends StatefulWidget {
  final Function(String name, dynamic value) onChanged;
  final String displayName;
  final String name;
  final List<dynamic> options;
  final String Function(dynamic)? labelsBuilder;
  final dynamic value;
  final List<String>? pathToField;
  final FormFieldState<dynamic>? state;

  const SelectFormFactory({
    Key? key,
    required this.onChanged,
    required this.displayName,
    required this.name,
    required this.options,
    this.labelsBuilder,
    this.value,
    this.pathToField,
    this.state,
  }) : super(key: key);

  @override
  _SelectFormState createState() => _SelectFormState();
}

class _SelectFormState extends State<SelectFormFactory> {
  late dynamic _value;

  @override
  void initState() {
    _value = widget.value;
    super.initState();
  }

  @override
  void didUpdateWidget(covariant SelectFormFactory oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (!const DeepCollectionEquality.unordered()
        .equals(_value, widget.value)) {
      _value = widget.value;
    }
  }

  @override
  Widget build(BuildContext context) {
    final Set<dynamic> items = {};
    items.addAll([...(widget.options.where((element) => element != null))]);
    if (!items.contains(_value)) {
      items.add(_value);
    }
    items.add(null);

    return BorderLabel(
      label: widget.displayName,
      child: DropdownSearch<dynamic>(
        mode: Mode.MENU,
        showSearchBox: true,
        items: items.toList(),
        onChanged: (dynamic data) {
          widget.onChanged(widget.name, data);
          widget.state?.didChange(data);
        },
        itemAsString: (dynamic rawRow) {
          if (widget.labelsBuilder != null) {
            return widget.labelsBuilder!(rawRow);
          }
          if (widget.pathToField != null && rawRow is Map) {
            return fieldFromPathToField(
                widget.pathToField!, rawRow.cast<String, dynamic>());
          }
          return rawRow?.toString() ?? "";
        },
        selectedItem: _value,
      ),
    );
  }
}
