import "dart:math";
import 'package:verttice_logic/verttice_logic.dart';
import "package:verttice_ui/src/const/icons.dart";
import "package:flutter/material.dart";
import "package:flutter/services.dart";
import "package:flutter_multi_formatter/flutter_multi_formatter.dart" as mf;
import "package:validators/validators.dart";
import 'package:verttice_ui/src/widgets/border_label.dart';

enum TextFieldInputFF { text, integer, email, double }

class TextFormFactory extends StatefulWidget {
  final String displayName;
  final String name;
  final TextFieldInputFF type;
  final dynamic value;
  final FormFieldState<dynamic>? state;
  final bool showStandartBorder;

  /// in order to format a string like this GHJ45GHJHN to GHJ-45-GHJHN use String like this: "###-00-#####"
  final String? textFormatter;
  final Function(String name, dynamic value) onChanged;

  const TextFormFactory({
    Key? key,
    required this.displayName,
    required this.name,
    required this.onChanged,
    this.showStandartBorder = true,
    this.type = TextFieldInputFF.text,
    this.value,
    this.textFormatter,
    this.state,
  }) : super(key: key);
  @override
  _TextFormState createState() => _TextFormState();
}

class _TextFormState extends State<TextFormFactory> {
  TextEditingController? _controller;
  String _value = "";
  int _baseOffset = 0;

  @override
  void initState() {
    super.initState();

    _value = DataClassUtils.dynamicToString(widget.value);
    if (widget.type == TextFieldInputFF.integer) {
      _value = widget.value == null || widget.value == ''
          ? ''
          : DataClassUtils.dynamicToInt(
              widget.value,
            ).toString();
    } else if (widget.type == TextFieldInputFF.double) {
      _value = widget.value == null || widget.value == ''
          ? ''
          : DataClassUtils.dynamicToDouble(
              widget.value,
            ).toString();
    }

    _controller = TextEditingController(
      text: _value,
    );

    _controller!.selection = TextSelection.fromPosition(
      TextPosition(
        offset: min(
          _baseOffset,
          _controller != null ? _controller!.text.length : 0,
        ),
      ),
    );

    _controller!.addListener(_listen);
  }

  @override
  void dispose() {
    _controller!.dispose();
    super.dispose();
  }

  @override
  void didUpdateWidget(covariant TextFormFactory oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (_value != widget.value) {
      _value = DataClassUtils.dynamicToString(widget.value);
      if (widget.type == TextFieldInputFF.integer) {
        _value = widget.value == null || widget.value == ''
            ? ''
            : DataClassUtils.dynamicToInt(
                widget.value,
              ).toString();
      } else if (widget.type == TextFieldInputFF.double) {
        _value = widget.value == null || widget.value == ''
            ? ''
            : DataClassUtils.dynamicToDouble(
                widget.value,
              ).toString();
      }

      _controller = TextEditingController(
        text: _value,
      );
      _controller!.selection = TextSelection.fromPosition(
        TextPosition(
          offset: min(
            _baseOffset,
            _controller != null ? _controller!.text.length : 0,
          ),
        ),
      );
      _controller!.addListener(_listen);
    }
  }

  void _listen() {
    setState(() {
      _baseOffset = _controller!.selection.baseOffset;
    });
  }

  Widget _form() {
    return TextFormField(
      cursorColor: Colors.amberAccent,
      onChanged: (val) {
        String newVal = val;
        if (widget.type == TextFieldInputFF.double) {
          newVal = val != "" ? int.parse(val).toString() : '';
        }

        if (widget.type == TextFieldInputFF.double) {
          newVal = val != ""
              ? double.parse(val.endsWith('.') ? "${val}0" : val).toString()
              : '';
        }

        setState(() {
          _value = newVal;
        });
        widget.onChanged(widget.name, newVal);

        widget.state?.didChange(newVal);
      },
      validator: (value) {
        if (widget.type == TextFieldInputFF.email &&
            value != null &&
            value != "" &&
            !isEmail(value)) {
          return "Please insert a valid email";
        }
        return null;
      },
      controller: _controller,
      keyboardType: TextInputType.text,
      inputFormatters: widget.type == TextFieldInputFF.double
          ? <TextInputFormatter>[
              TextInputFormatter.withFunction((oldValue, newValue) {
                try {
                  String text = newValue.text;
                  if (text.isEmpty) {
                    return newValue;
                  }
                  if (text.endsWith('.')) {
                    text += '0';
                  }
                  double.parse(text);
                  return newValue;
                } catch (e) {
                  return oldValue;
                }
              }),
            ]
          : widget.type == TextFieldInputFF.integer
              ? <TextInputFormatter>[
                  TextInputFormatter.withFunction((oldValue, newValue) {
                    try {
                      String text = newValue.text;
                      if (text.isEmpty) {
                        return newValue;
                      }

                      int.parse(text);
                      return newValue;
                    } catch (e) {
                      return oldValue;
                    }
                  }),
                ]
              : [
                  if (widget.textFormatter != null)
                    mf.MaskedInputFormatter(widget.textFormatter!)
                ],
      textDirection: TextDirection.ltr,
      autofocus: false,
      decoration: InputDecoration(
        label: widget.showStandartBorder ? null : Text(widget.displayName),
        suffixIcon: IconButton(
          onPressed: () {
            _controller!.clear();
            widget.onChanged(widget.name, "");
            widget.state?.didChange("");
          },
          icon: ThemedIcons.clear(
            iconStyle: IconStyle(
              color: Colors.redAccent,
            ),
          ),
        ),
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    return LayoutBuilder(builder: (context, constraints) {
      return SizedBox(
        width: constraints.maxWidth,
        child: widget.showStandartBorder
            ? BorderLabel(
                label: widget.displayName,
                child: _form(),
              )
            : _form(),
      );
    });
  }
}
