import 'package:verttice_logic/verttice_logic.dart';
import "package:verttice_ui/src/const/box_sizes.dart";
import "package:verttice_ui/src/const/icons.dart";
import "package:flutter/material.dart";
import "package:validators/validators.dart";
import 'package:verttice_ui/src/widgets/border_label.dart';
import 'package:verttice_ui/src/widgets/themed_widgets/themed_widgets_index.dart';

enum ListType {
  text,
  email,
}

class TextListFormFactory extends StatefulWidget {
  final String displayName;
  final String name;
  final Function(String name, dynamic value) onChanged;
  final List<String>? value;
  final ListType type;
  final FormFieldState<dynamic>? state;
  const TextListFormFactory({
    Key? key,
    this.value,
    required this.displayName,
    required this.name,
    required this.onChanged,
    required this.type,
    this.state,
  }) : super(key: key);

  @override
  _TextListState createState() => _TextListState();
}

class _TextListState extends State<TextListFormFactory> {
  List<String> _value = [];
  List<String> _newdData = [];
  late StateSetter _setState;

  @override
  void initState() {
    super.initState();
    _value = DataClassUtils.dynamicToStringList(widget.value);
    _newdData = DataClassUtils.dynamicToStringList(widget.value);
  }

  @override
  void didUpdateWidget(covariant TextListFormFactory oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (_value != widget.value) {
      _value = DataClassUtils.dynamicToStringList(widget.value);
      _newdData = DataClassUtils.dynamicToStringList(widget.value);
    }
  }

  @override
  Widget build(BuildContext context) {
    return BorderLabel(
      label: widget.displayName,
      child: SelectableText(_value.join(", ")),
      onPressed: () async {
        await showDialog(
          context: context,
          useSafeArea: true,
          builder: (BuildContext context) {
            return StatefulBuilder(builder: (BuildContext context, setState) {
              _setState = setState;

              return SimpleDialog(
                insetPadding: const EdgeInsets.all(6),
                contentPadding: const EdgeInsets.all(6),
                children: [
                  SizedBox(
                    height: 400,
                    width: 400,
                    child: Scaffold(
                      appBar: ThemedAppBar(
                        title: widget.displayName,
                        actions: [
                          _AddTextFieldListItem(
                            onChanged: (item) {
                              _setState(() {
                                _newdData.add(item);
                              });
                            },
                            type: widget.type,
                          ),
                          IconButton(
                            onPressed: () {
                              widget.onChanged(widget.name, _newdData);
                              widget.state?.didChange(_newdData);
                              Navigator.of(context).pop();
                            },
                            icon: ThemedIcons.check(),
                          ),
                        ],
                      ),
                      body: ListView.builder(
                        controller: ScrollController(),
                        itemCount: _newdData.length,
                        physics: const BouncingScrollPhysics(),
                        itemBuilder: (context, i) => _item(_newdData[i], i),
                      ),
                    ),
                  )
                ],
              );
            });
          },
        );

        setState(() {});
      },
    );
  }

  Widget _item(String item, int i) {
    return Row(
      children: [
        Expanded(
          child: _TextFieldListItem(
            type: widget.type,
            value: item,
            onChanged: (val) {
              _setState(() {
                _newdData = _newdData
                    .asMap()
                    .map((key, value) => MapEntry(key, key == i ? val : value))
                    .values
                    .map<String>((e) => e)
                    .toList();
              });
            },
          ),
        ),
        InkWell(
          onTap: () {
            _setState(() {
              _newdData.removeWhere((element) {
                return element == item;
              });
            });
          },
          child: CircleAvatar(
            backgroundColor: Colors.red,
            child: ThemedIcons.clear(
              iconStyle: IconStyle(
                color: Colors.white,
              ),
            ),
          ),
        )
      ],
    );
  }
}

class _TextFieldListItem extends StatefulWidget {
  final String value;
  final Function(String name) onChanged;
  final ListType type;

  const _TextFieldListItem({
    Key? key,
    required this.value,
    required this.onChanged,
    required this.type,
  }) : super(key: key);

  @override
  _TextFieldListItemState createState() => _TextFieldListItemState();
}

class _TextFieldListItemState extends State<_TextFieldListItem> {
  TextEditingController? _controller;
  String _value = "";
  String _newData = "";

  @override
  void initState() {
    super.initState();
    final data = DataClassUtils.dynamicToString(widget.value);
    _value = data;
    _newData = data;
    _controller = TextEditingController(
      text: data,
    );

    _controller!.selection = TextSelection.fromPosition(
        TextPosition(offset: _controller!.text.length));
  }

  @override
  void dispose() {
    _controller!.dispose();
    super.dispose();
  }

  @override
  void didUpdateWidget(covariant _TextFieldListItem oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (_value != widget.value) {
      final data = DataClassUtils.dynamicToString(widget.value);
      _value = data;
      _newData = data;
      _controller = TextEditingController(
        text: data,
      );
      _controller!.selection = TextSelection.fromPosition(
          TextPosition(offset: _controller!.text.length));
    }
  }

  @override
  Widget build(BuildContext context) {
    return Column(
      children: [
        sizedBoxH16,
        Row(
          children: [
            sizedBoxH8,
            Tooltip(
              message: "Edit",
              child: IconButton(
                icon: ThemedIcons.edit(),
                onPressed: () async {
                  await showDialog(
                    context: context,
                    builder: (BuildContext context) {
                      return StatefulBuilder(
                          builder: (BuildContext context, setState) {
                        return AlertDialog(
                            actions: [
                              TextButton(
                                  onPressed: () {
                                    setState(() {
                                      _controller =
                                          _controller = TextEditingController(
                                        text: _value,
                                      );
                                      _newData = _value;
                                    });
                                    Navigator.of(context).pop();
                                  },
                                  child: const Text("Cancel")),
                              TextButton(
                                onPressed: () {
                                  if (widget.type == ListType.email &&
                                      !isEmail(_newData)) {
                                    ThemedToast.error(
                                      context: context,
                                      message: "Please enter a valid email",
                                    );
                                  } else {
                                    widget.onChanged(_newData);

                                    setState(() {
                                      _controller =
                                          _controller = TextEditingController(
                                        text: _newData,
                                      );
                                      _controller!.text = _newData;
                                      _value = _newData;
                                    });
                                    Navigator.of(context).pop();
                                  }
                                },
                                child: const Text("Save"),
                              ),
                            ],
                            content: Builder(builder: (context) {
                              return SizedBox(
                                  height: 150,
                                  width: 400,
                                  child: Scaffold(
                                      body: Builder(builder: (context) {
                                    return TextField(
                                      autofocus: false,
                                      onChanged: (val) {
                                        setState(() {
                                          _newData = val;
                                        });
                                      },
                                      controller: _controller,
                                    );
                                  })));
                            }));
                      });
                    },
                  );
                },
              ),
            ),
            sizedBoxH8,
            SelectableText(
              widget.value,
              style: const TextStyle(
                overflow: TextOverflow.fade,
              ),
            ),
          ],
        ),
        sizedBoxH16,
        const Divider()
      ],
    );
  }
}

class _AddTextFieldListItem extends StatefulWidget {
  final String value;
  final Function(String name) onChanged;
  final ListType type;

  const _AddTextFieldListItem({
    Key? key,
    this.value = "",
    required this.onChanged,
    required this.type,
  }) : super(key: key);

  @override
  _AddTextFieldListItemState createState() => _AddTextFieldListItemState();
}

class _AddTextFieldListItemState extends State<_AddTextFieldListItem> {
  TextEditingController? _controller;
  String _newLine = "";

  @override
  void initState() {
    super.initState();

    _controller = TextEditingController(
      text: '',
    );
    _controller!.selection = TextSelection.fromPosition(
        TextPosition(offset: _controller!.text.length));
  }

  @override
  void dispose() {
    _controller!.dispose();
    super.dispose();
  }

  @override
  void didUpdateWidget(covariant _AddTextFieldListItem oldWidget) {
    super.didUpdateWidget(oldWidget);
  }

  void _cleanState() {
    _newLine = "";
  }

  @override
  Widget build(BuildContext context) {
    return IconButton(
      onPressed: () async {
        await showDialog(
          context: context,
          builder: (BuildContext context) {
            return StatefulBuilder(builder: (BuildContext context, setState) {
              return AlertDialog(
                  actions: [
                    TextButton(
                      child: const Text("Cancel"),
                      onPressed: () {
                        _cleanState();
                        setState(() {});
                        Navigator.of(context).pop();
                      },
                    ),
                    TextButton(
                      child: const Text("Save"),
                      onPressed: () {
                        String temp = _newLine;
                        if (widget.type == ListType.email && !isEmail(temp)) {
                          ThemedToast.error(
                            context: context,
                            message: "Please enter a valid email",
                          );
                        } else {
                          setState(() {
                            _newLine = "";
                          });
                          widget.onChanged(temp);

                          Navigator.of(context).pop();
                        }
                      },
                    ),
                  ],
                  content: Builder(builder: (context) {
                    return SizedBox(
                        height: 150,
                        width: 400,
                        child: Scaffold(
                            body: Builder(builder: (BuildContext context) {
                          return TextField(
                            autofocus: false,
                            onChanged: (val) {
                              setState(() {
                                _newLine = val;
                              });
                            },
                            controller: _controller,
                          );
                        })));
                  }));
            });
          },
        );
      },
      icon: ThemedIcons.add(),
    );
  }
}
