import "dart:typed_data";
import 'package:verttice_logic/verttice_logic.dart';
import 'package:verttice_ui/src/const/box_sizes.dart';
import 'package:verttice_ui/src/const/icons.dart';
import "package:flutter/material.dart";
import "package:photo_view/photo_view.dart";
import 'package:verttice_ui/src/widgets/download_button.dart';
import 'package:verttice_ui/src/widgets/themed_widgets/themed_widgets_index.dart';

class ImageViewerFullPage extends StatefulWidget {
  final String? url;
  final String? asset;
  final Uint8List? data;
  final String imageID;
  const ImageViewerFullPage({
    Key? key,
    this.url,
    this.asset,
    this.data,
    required this.imageID,
  }) : super(key: key);

  @override
  ImageViewerFullPageState createState() => ImageViewerFullPageState();
}

class ImageViewerFullPageState extends State<ImageViewerFullPage> {
  int _quarterTurns = 0;
  @override
  Widget build(BuildContext context) {
    ImageProvider<Object> provider;
    late DownloadButton downloadWidget;
    if (widget.url != null) {
      provider = NetworkImage(widget.url!);
      downloadWidget = DownloadButton(
        url: widget.url!,
        fileType: VFileType.png,
      );
    } else if (widget.data != null) {
      provider = MemoryImage(widget.data!);
      downloadWidget = DownloadButton(
        fileData: widget.data!,
        fileType: VFileType.png,
      );
    } else {
      provider = AssetImage(widget.asset!);
      downloadWidget = DownloadButton(
        asset: widget.asset!,
        fileType: VFileType.png,
      );
    }
    return Scaffold(
      appBar: ThemedAppBar(
        actions: [
          sizedBoxW8,
          if (widget.url != null)
            CircleAvatar(
              backgroundColor: Colors.lightBlue,
              child: downloadWidget,
            ),
          sizedBoxW8,
          CircleAvatar(
            backgroundColor: Colors.lightBlue,
            child: IconButton(
              onPressed: () {
                setState(() {
                  _quarterTurns =
                      _quarterTurns + 1 == 4 ? 0 : _quarterTurns + 1;
                });
              },
              icon: ThemedIcons.refresh(),
            ),
          ),
          sizedBoxW8,
        ],
      ),
      body: Container(
        color: Colors.blueGrey[700]!.withOpacity(0.5),
        child: RotatedBox(
          quarterTurns: _quarterTurns,
          child: PhotoView(
            onTapDown: (
              BuildContext context,
              TapDownDetails details,
              PhotoViewControllerValue controller,
            ) {
              Navigator.pop(context);
            },
            backgroundDecoration: const BoxDecoration(
              color: Colors.transparent,
            ),
            imageProvider: provider,
            heroAttributes: PhotoViewHeroAttributes(tag: widget.imageID),
            initialScale: PhotoViewComputedScale.contained,
          ),
        ),
      ),
    );
  }
}
