import "package:flutter/material.dart";
import 'package:verttice_ui/verttice_ui.dart';

import 'pdf_viewer_full_page_network.dart';

class PdfViewerStorage extends StatefulWidget {
  final String path;
  final double? height;
  final String? diaplyName;

  const PdfViewerStorage({
    Key? key,
    required this.path,
    this.height,
    this.diaplyName,
  }) : super(key: key);

  @override
  _PdfViewerState createState() => _PdfViewerState();
}

class _PdfViewerState extends State<PdfViewerStorage> {
  late String _path;
  late Future<String?> _fetchData;
  bool _loaded = false;

  @override
  void initState() {
    _path = widget.path;
    _fetchData = VertticeUIConfig.instance.getStorageDownloadURL(path: _path);

    super.initState();
  }

  @override
  void didUpdateWidget(covariant PdfViewerStorage oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (_path != widget.path) {
      _path = widget.path;
      _loaded = false;
      _fetchData = VertticeUIConfig.instance.getStorageDownloadURL(path: _path);
    }
  }

  @override
  Widget build(BuildContext context) {
    return FutureBuilder<String?>(
        future: _fetchData,
        builder: (BuildContext context, AsyncSnapshot<String?> snapdata) {
          return SnapBuilder<String?>(
              height: widget.height,
              snapshot: snapdata,
              loaded: _loaded,
              erroBuilder: (dynamic error) {
                return Center(
                  child: CircleAvatar(
                    backgroundColor: Colors.red,
                    child: ThemedIcons.fileUnknow(
                      iconStyle: IconStyle(
                        color: Colors.white,
                        size: 35,
                      ),
                    ),
                  ),
                );
              },
              loadingBuilder: () => const ThemedProgress(),
              successBuilder: (String? data) {
                _loaded = true;
                return _body(
                  context,
                  data ?? '',
                );
              });
        });
  }

  Widget _body(
    BuildContext context,
    String downloadedFile,
  ) {
    return SizedBox(
      height: widget.height ?? MediaQuery.of(context).size.height / 2.5,
      child: InkWell(
        onTap: () async {
          Navigator.push(
            context,
            MaterialPageRoute(
              builder: (context) => PdfViewerFullPage(
                url: downloadedFile,
                fileName: widget.diaplyName != null
                    ? widget.diaplyName!.replaceAll(' ', '_')
                    : null,
              ),
            ),
          );
        },
        child: Container(
          decoration: const BoxDecoration(
            borderRadius: BorderRadius.all(
              Radius.circular(5.0),
            ),
            image: DecorationImage(
              image: AssetImage(
                VertticeUIAssets.pdf,
                package: 'verttice_ui',
              ),
              fit: BoxFit.cover,
            ),
          ),
          child: Align(
            alignment: Alignment.bottomCenter,
            child: SelectableText(
              widget.diaplyName ?? "PDF",
              style: const TextStyle(
                color: Colors.black,
              ),
            ),
          ),
        ),
      ),
    );
  }
}
