import 'package:chewie/chewie.dart';
import 'package:verttice_logic/verttice_logic.dart';
import "package:flutter/material.dart";
import 'package:verttice_ui/verttice_ui.dart';
import 'package:video_player/video_player.dart';

class VideoViewerFullPage extends StatefulWidget {
  final String url;
  final double? height;
  final bool showAppBar;
  const VideoViewerFullPage({
    Key? key,
    required this.url,
    this.height,
    this.showAppBar = true,
  }) : super(key: key);

  @override
  _VideoViewerFullPageState createState() => _VideoViewerFullPageState();
}

class _VideoViewerFullPageState extends State<VideoViewerFullPage> {
  late VideoPlayerController _videoPlayerController;
  ChewieController? _chewieController;
  late String _url;

  @override
  void initState() {
    super.initState();
    _url = widget.url;
    _initVideoPlayer();
  }

  @override
  void dispose() {
    super.dispose();
    _videoPlayerController.dispose();
    _chewieController?.dispose();
  }

  @override
  void didUpdateWidget(covariant VideoViewerFullPage oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (_url != widget.url) {
      _url = widget.url;
      _initVideoPlayer();
    }
  }

  Future<void> _initVideoPlayer() async {
    _videoPlayerController = VideoPlayerController.network(_url);
    await _videoPlayerController.initialize();
    _chewieController = ChewieController(
      videoPlayerController: _videoPlayerController,
      looping: true,
    );
    setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      backgroundColor: Colors.grey[850],
      appBar: !widget.showAppBar
          ? null
          : AppBar(
              actions: [
                DownloadButton(
                  url: widget.url,
                  fileType: VFileType.mp4,
                ),
                sizedBoxH16,
              ],
              backgroundColor: Colors.black,
            ),
      floatingActionButtonLocation: FloatingActionButtonLocation.endTop,
      floatingActionButton: widget.showAppBar
          ? null
          : CircleAvatar(
              child: DownloadButton(
                url: widget.url,
                fileType: VFileType.mp4,
              ),
            ),
      body: SizedBox(
        height: widget.height,
        child: Builder(builder: (BuildContext context) {
          if (_chewieController == null) {
            return const Center(child: ThemedProgress());
          }

          return Center(
            child: Container(
              decoration: BoxDecoration(
                borderRadius: BorderRadius.circular(10.0),
              ),
              child: SafeArea(
                child: Chewie(
                  controller: _chewieController!,
                ),
              ),
            ),
          );
        }),
      ),
    );
  }
}
