import "package:flutter/material.dart";
import 'package:verttice_ui/verttice_ui.dart';

import 'video_viewer_full_page_network.dart';

class VideoViewerNetwork extends StatefulWidget {
  final String url;
  final double? height;

  const VideoViewerNetwork({
    Key? key,
    required this.url,
    this.height,
  }) : super(key: key);
  @override
  State createState() => _VideoViewerNetworkState();
}

class _VideoViewerNetworkState extends State<VideoViewerNetwork> {
  late String _url;
  late Future<String?> _fetchData;

  @override
  void initState() {
    super.initState();
    _url = widget.url;

    _fetchData = VertticeUIConfig.instance.getStorageDownloadURL(path: _url);
  }

  @override
  void didUpdateWidget(covariant VideoViewerNetwork oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (_url != widget.url) {
      _url = widget.url;
      _fetchData = VertticeUIConfig.instance.getStorageDownloadURL(path: _url);
    }
  }

  @override
  void dispose() {
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return FutureBuilder<String?>(
      future: _fetchData,
      builder: (BuildContext context, AsyncSnapshot<String?> snapdata) {
        return SnapBuilder<String?>(
          height: widget.height,
          snapshot: snapdata,
          erroBuilder: (dynamic error) => Center(
            child: CircleAvatar(
              backgroundColor: Colors.red,
              child: ThemedIcons.fileUnknow(
                iconStyle: IconStyle(
                  color: Colors.white,
                  size: 35,
                ),
              ),
            ),
          ),
          loadingBuilder: () => const ThemedProgress(),
          successBuilder: (String? data) {
            return _body(context, data ?? '');
          },
        );
      },
    );
  }

  Widget _body(BuildContext context, String downloadURL) {
    return SizedBox(
      height: widget.height ?? MediaQuery.of(context).size.height / 2.5,
      child: InkWell(
        onTap: () {
          Navigator.push(
            context,
            MaterialPageRoute(
              builder: (context) => VideoViewerFullPage(url: downloadURL),
            ),
          );
        },
        child: Container(
          decoration: const BoxDecoration(
            borderRadius: BorderRadius.all(
              Radius.circular(5.0),
            ),
            image: DecorationImage(
              image: AssetImage(
                VertticeUIAssets.video,
                package: 'verttice_ui',
              ),
              fit: BoxFit.cover,
            ),
          ),
          child: const Align(
            alignment: Alignment.bottomCenter,
            child: SelectableText(
              "Click to watch",
              style: TextStyle(
                color: Colors.white,
              ),
            ),
          ),
        ),
      ),
    );
  }
}
