import "package:flutter/material.dart";
import 'package:verttice_ui/verttice_ui.dart';

import 'video_viewer_full_page_network.dart';

class VideoViewerStorage extends StatefulWidget {
  final String path;
  final double? height;

  const VideoViewerStorage({
    Key? key,
    required this.path,
    this.height,
  }) : super(key: key);
  @override
  State createState() => _VideoViewerStorageState();
}

class _VideoViewerStorageState extends State<VideoViewerStorage> {
  late String _path;
  late Future<String?> _fetchData;

  @override
  void initState() {
    super.initState();
    _path = widget.path;

    _fetchData = VertticeUIConfig.instance.getStorageDownloadURL(path: _path);
  }

  @override
  void didUpdateWidget(covariant VideoViewerStorage oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (_path != widget.path) {
      _path = widget.path;
      _fetchData = VertticeUIConfig.instance.getStorageDownloadURL(path: _path);
    }
  }

  @override
  void dispose() {
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return FutureBuilder<String?>(
        future: _fetchData,
        builder: (BuildContext context, AsyncSnapshot<String?> snapdata) {
          return SnapBuilder<String?>(
              height: widget.height,
              snapshot: snapdata,
              erroBuilder: (dynamic error) => Center(
                    child: CircleAvatar(
                      backgroundColor: Colors.red,
                      child: ThemedIcons.fileUnknow(
                        iconStyle: IconStyle(
                          color: Colors.white,
                          size: 35,
                        ),
                      ),
                    ),
                  ),
              loadingBuilder: () => const ThemedProgress(),
              successBuilder: (String? data) {
                return _body(context, data ?? '');
              });
        });
  }

  Widget _body(BuildContext context, String downloadURL) {
    return SizedBox(
      height: widget.height ?? MediaQuery.of(context).size.height / 2.5,
      child: InkWell(
        onTap: () {
          Navigator.push(
            context,
            MaterialPageRoute(
              builder: (context) => VideoViewerFullPage(url: downloadURL),
            ),
          );
        },
        child: Container(
          decoration: const BoxDecoration(
            borderRadius: BorderRadius.all(
              Radius.circular(5.0),
            ),
            image: DecorationImage(
              image: AssetImage(
                VertticeUIAssets.video,
                package: 'verttice_ui',
              ),
              fit: BoxFit.cover,
            ),
          ),
          child: const Align(
            alignment: Alignment.bottomCenter,
            child: SelectableText(
              "Click to watch",
              style: TextStyle(
                color: Colors.white,
              ),
            ),
          ),
        ),
      ),
    );
  }
}
