import "package:flutter/material.dart";
import "package:google_maps_flutter/google_maps_flutter.dart";

class MapCoordinatesPage extends StatefulWidget {
  final double lat;
  final double lng;
  final String? placeName;
  final double mapHeight;
  final double mapWidth;

  const MapCoordinatesPage({
    Key? key,
    required this.lat,
    required this.lng,
    required this.mapHeight,
    required this.mapWidth,
    this.placeName,
  }) : super(key: key);

  @override
  State<StatefulWidget> createState() => _MapCoordinatesBodyState();
}

class _MapCoordinatesBodyState extends State<MapCoordinatesPage> {
  _MapCoordinatesBodyState();
  late double _lat;
  late double _lng;
  String? _placeName;

  Set<Marker> _markers = <Marker>{};

  GoogleMapController? _mapController;

  @override
  void initState() {
    super.initState();

    _lat = widget.lat;
    _lng = widget.lng;
    _placeName = widget.placeName;
    final markers = <Marker>{};
    markers.add(Marker(
      draggable: false,
      markerId: MarkerId("${_lat.toString()}${_lng.toString()}"),
      position: LatLng(_lat, _lng),
      infoWindow: InfoWindow(title: _placeName),
    ));
    _markers = markers;
  }

  @override
  void didUpdateWidget(covariant MapCoordinatesPage oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (widget.lat != _lat || widget.lng != _lng) {
      _lat = widget.lat;
      _lng = widget.lng;
      _placeName = widget.placeName;
      final markers = <Marker>{};
      markers.add(Marker(
        draggable: false,
        markerId: MarkerId("${_lat.toString()}${_lng.toString()}"),
        position: LatLng(_lat, _lng),
        infoWindow: InfoWindow(title: _placeName),
      ));
      _markers = markers;
      _mapController?.animateCamera(
        CameraUpdate.newCameraPosition(
          CameraPosition(
            target: LatLng(_lat, _lng),
            zoom: 13.0,
          ),
        ),
      );
    }
  }

  @override
  Widget build(BuildContext context) {
    return Column(
      mainAxisAlignment: MainAxisAlignment.center,
      crossAxisAlignment: CrossAxisAlignment.center,
      children: [
        if (_placeName != null)
          Center(
            child: Padding(
              padding: const EdgeInsets.symmetric(horizontal: 7, vertical: 7),
              child: SelectableText(_placeName!),
            ),
          ),
        Padding(
          padding: const EdgeInsets.all(18.0),
          child: Center(
            child: SizedBox(
              height: widget.mapHeight,
              width: widget.mapWidth,
              child: GoogleMap(
                trafficEnabled: true,
                myLocationButtonEnabled: true,
                myLocationEnabled: true,
                onMapCreated: _onMapCreated,
                initialCameraPosition: CameraPosition(
                  target: LatLng(_lat, _lng),
                  zoom: 15.5,
                ),
                markers: _markers,
              ),
            ),
          ),
        ),
      ],
    );
  }

  void _onMapCreated(GoogleMapController controller) async {
    _mapController = controller;
  }
}
