import 'package:flutter/foundation.dart';
import "package:flutter/material.dart";
import 'package:verttice_ui/src/widgets/themed_widgets/themed_widgets_index.dart';

class SnapBuilder<T> extends StatelessWidget {
  final AsyncSnapshot<T> snapshot;
  final Widget Function(dynamic error)? erroBuilder;
  final Widget Function(T data) successBuilder;
  final Widget Function()? loadingBuilder;
  final Widget Function()? nullBuilder;
  final bool loaded;
  final double? width;
  final double? height;
  const SnapBuilder({
    Key? key,
    required this.snapshot,
    required this.successBuilder,
    this.erroBuilder,
    this.nullBuilder,
    this.loaded = false,
    this.loadingBuilder,
    this.width,
    this.height,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    if (loaded) {
      return successBuilder(snapshot.data!);
    } else if (snapshot.connectionState == ConnectionState.waiting) {
      return loadingBuilder != null
          ? loadingBuilder!()
          : const ThemedMessage(showLoading: true);
    } else if (snapshot.hasData) {
      if (snapshot.data == null) {
        if (nullBuilder != null) {
          return nullBuilder!();
        }
        return ThemedMessage(
          message: "Nothing to show...",
          height: height,
          width: width,
        );
      } else {
        return successBuilder(snapshot.data!);
      }
    } else if (snapshot.hasError) {
      if (kDebugMode) {
        print(snapshot.error);
      }
      if (erroBuilder != null) {
        return erroBuilder!(snapshot.error);
      }
      return ThemedMessage(
        message: "Failed to load data...",
        height: height,
        width: width,
      );
    }

    return ThemedMessage(
      message: "Nothing to show...",
      height: height,
      width: width,
    );
  }
}
