import "package:flutter/material.dart";
import 'package:provider/provider.dart';
import 'package:verttice_ui/src/state/verttice_ui_state.dart';

class ThemedButton extends StatelessWidget {
  final Color? color;
  final Function() onPressed;
  final double borderRadiusPx;
  final bool offMode;
  final List<BoxShadow>? boxShadow;
  final Widget? child;
  final EdgeInsets? padding;
  final EdgeInsets? margin;

  const ThemedButton({
    Key? key,
    this.color,
    required this.onPressed,
    required this.child,
    this.borderRadiusPx = 25.0,
    this.offMode = false,
    this.boxShadow,
    this.padding = const EdgeInsets.all(5),
    this.margin,
  }) : super(key: key);
  @override
  @override
  Widget build(BuildContext context) {
    return Consumer<VertticeUIState>(builder: (context, value, _child) {
      return Container(
        decoration: BoxDecoration(
          borderRadius: BorderRadius.circular(borderRadiusPx),
        ),
        child: InkWell(
            onTap: onPressed,
            child: Container(
              decoration: BoxDecoration(
                color: offMode ? Colors.grey[400] : (color ?? value.themeColor),
                borderRadius: BorderRadius.circular(borderRadiusPx),
                boxShadow: boxShadow ??
                    [
                      BoxShadow(
                        color: Colors.grey[300]!,
                        blurRadius: 1.5,
                        offset: const Offset(
                          1.0,
                          1.0,
                        ),
                      )
                    ],
              ),
              child: Center(
                child: Container(
                  padding: padding,
                  margin: margin,
                  child: child,
                ),
              ),
            )),
      );
    });
  }
}
