import "package:flutter/material.dart";
import 'package:provider/provider.dart';
import 'package:verttice_ui/src/state/verttice_ui_state.dart';

import 'themed_progress.dart';

class ThemedTextButton extends StatelessWidget {
  final String text;
  final Color? textColor;
  final Function() onPressed;
  final double borderRadiusPx;
  final bool offMode;
  final List<BoxShadow>? boxShadow;
  final EdgeInsets? padding;
  final EdgeInsets? margin;
  final bool showLoading;
  final Color? color;

  const ThemedTextButton(
    this.text, {
    Key? key,
    required this.onPressed,
    this.borderRadiusPx = 25.0,
    this.offMode = false,
    this.boxShadow,
    this.padding = const EdgeInsets.all(5),
    this.margin,
    this.showLoading = false,
    this.textColor,
    this.color,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Consumer<VertticeUIState>(
      builder: (context, value, _child) {
        return Container(
          decoration: BoxDecoration(
            borderRadius: BorderRadius.circular(borderRadiusPx),
          ),
          child: InkWell(
            onTap: onPressed,
            child: Container(
              decoration: BoxDecoration(
                color: offMode ? Colors.grey[400] : (color ?? value.themeColor),
                borderRadius: BorderRadius.circular(borderRadiusPx),
                boxShadow: boxShadow ??
                    [
                      BoxShadow(
                        color: Colors.grey[350]!,
                        blurRadius: 1.5,
                        offset: const Offset(
                          1.0,
                          1.0,
                        ),
                      )
                    ],
              ),
              child: Center(
                child: Container(
                  padding: padding,
                  margin: margin,
                  child: Row(
                    crossAxisAlignment: CrossAxisAlignment.center,
                    mainAxisAlignment: MainAxisAlignment.center,
                    children: [
                      Text(
                        text,
                        style: TextStyle(
                          color: textColor ?? value.themeTextColor,
                        ),
                      ),
                      if (showLoading)
                        const Padding(
                          padding: EdgeInsets.symmetric(horizontal: 15),
                          child: ThemedProgress(),
                        )
                    ],
                  ),
                ),
              ),
            ),
          ),
        );
      },
    );
  }
}
